#!/usr/bin/python
#

import argparse
import json
import random
import string
import subprocess
import sys


class argonRunner:
    GOODCHARS = string.ascii_letters + string.digits

    def __init__(self, args):
        self.exe = args.exe
        self.mnsaltlen = args.mnsaltlen
        self.mnpwlen = args.mnpwlen
        self.mndgstlen = args.mndgstlen
        self.mnmem = args.mnmem
        self.mniters = args.mniters
        self.mxsaltlen = args.mxsaltlen
        self.mxpwlen = args.mxpwlen
        self.mxdgstlen = args.mxdgstlen
        self.mxmem = args.mxmem
        self.mxiters = args.mxiters
        self.encoded = args.encoded
        self.rng = random.SystemRandom()
        self.version = args.version
        self.construct = args.construct
        self.maxcount = args.n
        self.count = 0

    def _runOnce(self, passwd, salt, dgst_len, maxmem, iters):
        argv = [
            self.exe,
            salt.encode("ascii"),
            "-t",
            "{:2d}".format(iters),
            "-m",
            "{:2d}".format(maxmem),
            "-l",
            "{:3d}".format(dgst_len),
            "-v",
            self.version,
        ]

        if self.encoded:
            argv.append("-e")
            mode = "crypt"
        else:
            argv.append("-r")
            mode = "raw"
        if self.construct == "argon2i":
            argv.append("-i")
        elif self.construct == "argon2d":
            argv.append("-d")
        elif self.construct == "argon2id":
            argv.append("-id")
        p = subprocess.Popen(
            argv, stdin=subprocess.PIPE, stdout=subprocess.PIPE
        )
        out, err = p.communicate(passwd.encode("ascii"))
        return dict(
            passwd=passwd,
            salt=salt,
            dgst_len=dgst_len,
            maxmem=2**maxmem,
            iters=iters,
            mode=mode,
            pwhash=out.decode("ascii").rstrip(),
            construct=self.construct,
        )

    def _genSalt(self):
        sltln = self.rng.randint(self.mnsaltlen, self.mxsaltlen)
        chrs = [self.rng.choice(self.GOODCHARS) for x in range(sltln)]
        return "".join(chrs)

    def _genPw(self):
        pwln = self.rng.randint(self.mnpwlen, self.mxpwlen)
        chrs = [self.rng.choice(self.GOODCHARS) for x in range(pwln)]
        return "".join(chrs)

    def __next__(self):
        if self.count >= self.maxcount:
            raise StopIteration
        psw = self._genPw()
        slt = self._genSalt()
        mem = self.rng.randint(self.mnmem, self.mxmem)
        iters = self.rng.randint(self.mniters, self.mxiters)
        dgstln = self.rng.randint(self.mndgstlen, self.mxdgstlen)
        rs = self._runOnce(psw, slt, dgstln, mem, iters)
        self.count += 1
        return rs

    def __iter__(self):
        return self

    next = __next__


if __name__ == "__main__":
    p = argparse.ArgumentParser()
    p.add_argument("-x", "--executable", dest="exe", required=True)
    p.add_argument(
        "-c", "--construction", dest="construct", type=str, default="argon2i"
    )
    p.add_argument("-v", "--version", dest="version", type=str, default="13")
    p.add_argument(
        "-e",
        "--encoded",
        dest="encoded",
        default=False,
        action="store_true",
    )
    p.add_argument(
        "-s", "--min-salt-len", dest="mnsaltlen", type=int, default=8
    )
    p.add_argument(
        "-S", "--max-salt-len", dest="mxsaltlen", type=int, default=8
    )
    p.add_argument(
        "-p", "--min-password-len", dest="mnpwlen", type=int, default=16
    )
    p.add_argument(
        "-P", "--max-password-len", dest="mxpwlen", type=int, default=16
    )
    p.add_argument(
        "-l", "--min-digest-len", dest="mndgstlen", type=int, default=64
    )
    p.add_argument(
        "-L", "--max-digest-len", dest="mxdgstlen", type=int, default=64
    )
    p.add_argument(
        "-m", "--min-memory-exponent", dest="mnmem", type=int, default=16
    )
    p.add_argument(
        "-M", "--max-memory-exponent", dest="mxmem", type=int, default=16
    )
    p.add_argument(
        "-t", "--min-time-opscount", dest="mniters", type=int, default=3
    )
    p.add_argument(
        "-T", "--max-time-opscount", dest="mxiters", type=int, default=3
    )
    p.add_argument("-n", "--count", dest="n", type=int, default=10)
    p.add_argument(
        "-w",
        "--output",
        dest="outfile",
        default=sys.stdout,
        type=argparse.FileType("w"),
    )

    args = p.parse_args()

    res = [x for x in argonRunner(args)]

    json.dump(res, args.outfile, indent=2, separators=(",", ": "))



#! /usr/bin/env python3

import glob
import os
import uuid

dirs = set()

tlv1 = ""
for file in glob.iglob("src/libsodium/**/*.c", recursive=True):
    file = file.replace("/", "\\")
    tlv1 = tlv1 + '    <ClCompile Include="..\\..\\{}" />\r\n'.format(file)

tlv2 = ""
for file in glob.iglob("src/libsodium/**/*.h", recursive=True):
    file = file.replace("/", "\\")
    tlv2 = tlv2 + '    <ClInclude Include="..\\..\\{}" />\r\n'.format(file)

tlf1 = ""
for file in glob.iglob("src/libsodium/**/*.c", recursive=True):
    file = file.replace("/", "\\")
    tlf1 = tlf1 + '    <ClCompile Include="..\\..\\{}">\r\n'.format(file)
    tlf1 = tlf1 + "      <Filter>Source Files</Filter>\r\n"
    tlf1 = tlf1 + "    </ClCompile>\r\n"

tlf2 = ""
for file in glob.iglob("src/libsodium/**/*.h", recursive=True):
    file = file.replace("/", "\\")
    tlf2 = tlf2 + '    <ClInclude Include="..\\..\\{}">\r\n'.format(file)
    tlf2 = tlf2 + "      <Filter>Header Files</Filter>\r\n"
    tlf2 = tlf2 + "    </ClInclude>\r\n"

v1 = ""
for file in glob.iglob("src/libsodium/**/*.c", recursive=True):
    file = file.replace("/", "\\")
    v1 = v1 + '    <ClCompile Include="..\\..\\..\\..\\{}" />\r\n'.format(file)

v2 = ""
for file in glob.iglob("src/libsodium/**/*.h", recursive=True):
    file = file.replace("/", "\\")
    v2 = v2 + '    <ClInclude Include="..\\..\\..\\..\\{}" />\r\n'.format(file)

f1 = ""
for file in glob.iglob("src/libsodium/**/*.c", recursive=True):
    basedir = os.path.dirname(file).replace("src/libsodium/", "")
    t = basedir
    while t != "":
        dirs.add(t)
        t = os.path.dirname(t)
    basedir = basedir.replace("/", "\\")
    file = file.replace("/", "\\")
    f1 = f1 + '    <ClCompile Include="..\\..\\..\\..\\{}">\r\n'.format(file)
    f1 = f1 + "      <Filter>{}</Filter>\r\n".format(basedir)
    f1 = f1 + "    </ClCompile>\r\n"

f2 = ""
for file in glob.iglob("src/libsodium/**/*.h", recursive=True):
    basedir = os.path.dirname(file).replace("src/libsodium/", "")
    t = basedir
    while t != "":
        dirs.add(t)
        t = os.path.dirname(t)
    basedir = basedir.replace("/", "\\")
    file = file.replace("/", "\\")
    f2 = f2 + '    <ClInclude Include="..\\..\\..\\..\\{}">\r\n'.format(file)
    f2 = f2 + "      <Filter>{}</Filter>\r\n".format(basedir)
    f2 = f2 + "    </ClInclude>\r\n"

fd = ""
dirs = list(dirs)
dirs.sort()
for dir in dirs:
    dir = dir.replace("/", "\\")
    uid = uuid.uuid3(uuid.UUID(bytes=b"LibSodiumMSVCUID"), dir)
    fd = fd + '    <Filter Include="{}">\r\n'.format(dir)
    fd = fd + "      <UniqueIdentifier>{{{}}}</UniqueIdentifier>\r\n".format(uid)
    fd = fd + "    </Filter>\r\n"


def get_project_configurations(vs_version):
    projconfig = ""
    configs = [
        "DebugDLL",
        "ReleaseDLL",
        "DebugLIB",
        "ReleaseLIB",
        "DebugLTCG",
        "ReleaseLTCG",
    ]
    platforms = ["Win32", "x64"]
    # add arm64 platform only for v142+ toolchain (VS2019+)
    if vs_version >= 142:
        platforms.append("ARM64")
    for config in configs:
        for platform in platforms:
            projconfig = (
                projconfig
                + '    <ProjectConfiguration Include="{}|{}">\r\n'.format(
                    config, platform
                )
            )
            projconfig = (
                projconfig
                + "      <Configuration>{}</Configuration>\r\n".format(config)
            )
            projconfig = projconfig + "      <Platform>{}</Platform>\r\n".format(
                platform
            )
            projconfig = projconfig + "    </ProjectConfiguration>\r\n"
    return projconfig


def apply_template(tplfile, outfile, sbox):
    tpl = ""
    with open(tplfile, "rb") as fd:
        tpl = fd.read()
    for s in sbox.keys():
        tpl = tpl.replace(
            str.encode("{{" + s + "}}", "utf8"), str.encode(str.strip(sbox[s]), "utf8")
        )

    with open(outfile, "wb") as fd:
        fd.write(tpl)


sbox = {
    "tlv1": tlv1,
    "tlv2": tlv2,
    "tlf1": tlf1,
    "tlf2": tlf2,
    "v1": v1,
    "v2": v2,
    "f1": f1,
    "f2": f2,
    "fd": fd,
}

sd = os.path.dirname(os.path.realpath(__file__))

apply_template(
    sd + "/tl_libsodium.vcxproj.filters.tpl",
    "ci/appveyor/libsodium.vcxproj.filters",
    sbox,
)

sbox.update({"platform": "v140"})
sbox.update({"configurations": get_project_configurations(140)})
apply_template(sd + "/tl_libsodium.vcxproj.tpl", "ci/appveyor/libsodium.vcxproj", sbox)

apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2026/libsodium/libsodium.vcxproj.filters",
    sbox,
)
apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2022/libsodium/libsodium.vcxproj.filters",
    sbox,
)
apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2019/libsodium/libsodium.vcxproj.filters",
    sbox,
)
apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2017/libsodium/libsodium.vcxproj.filters",
    sbox,
)
apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2015/libsodium/libsodium.vcxproj.filters",
    sbox,
)
apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2013/libsodium/libsodium.vcxproj.filters",
    sbox,
)
apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2012/libsodium/libsodium.vcxproj.filters",
    sbox,
)
apply_template(
    sd + "/libsodium.vcxproj.filters.tpl",
    "builds/msvc/vs2010/libsodium/libsodium.vcxproj.filters",
    sbox,
)

sbox.update({"platform": "v145"})
sbox.update({"configurations": get_project_configurations(145)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2026/libsodium/libsodium.vcxproj",
    sbox,
)

sbox.update({"platform": "v143"})
sbox.update({"configurations": get_project_configurations(143)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2022/libsodium/libsodium.vcxproj",
    sbox,
)

sbox.update({"platform": "v142"})
sbox.update({"configurations": get_project_configurations(142)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2019/libsodium/libsodium.vcxproj",
    sbox,
)

sbox.update({"platform": "v141"})
sbox.update({"configurations": get_project_configurations(141)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2017/libsodium/libsodium.vcxproj",
    sbox,
)

sbox.update({"platform": "v140"})
sbox.update({"configurations": get_project_configurations(140)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2015/libsodium/libsodium.vcxproj",
    sbox,
)

sbox.update({"platform": "v120"})
sbox.update({"configurations": get_project_configurations(120)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2013/libsodium/libsodium.vcxproj",
    sbox,
)

sbox.update({"platform": "v110"})
sbox.update({"configurations": get_project_configurations(110)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2012/libsodium/libsodium.vcxproj",
    sbox,
)

sbox.update({"platform": "v100"})
sbox.update({"configurations": get_project_configurations(100)})
apply_template(
    sd + "/libsodium.vcxproj.tpl",
    "builds/msvc/vs2010/libsodium/libsodium.vcxproj",
    sbox,
)



#!/usr/bin/env python
# Copyright 2013 Donald Stufft and individual contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import errno
import functools
import glob
import os
import os.path
import platform
import shutil
import subprocess
import sys
from sysconfig import get_config_vars

from setuptools import Distribution, setup
from setuptools.command.build_clib import build_clib as _build_clib
from setuptools.command.build_ext import build_ext as _build_ext


if platform.python_implementation() == "PyPy":
    if sys.pypy_version_info < (2, 6):
        raise RuntimeError(
            "PyNaCl is not compatible with PyPy < 2.6. Please "
            "upgrade PyPy to use this library."
        )


def here(*paths):
    return os.path.relpath(os.path.join(*paths))


def abshere(*paths):
    return os.path.abspath(here(*paths))


sodium = functools.partial(here, "src/libsodium/src/libsodium")


sys.path.insert(0, abshere("src"))


import nacl  # noqa


def use_system():
    install_type = os.environ.get("SODIUM_INSTALL")

    if install_type == "system":
        # If we are forcing system installs, don't compile the bundled one
        return True
    else:
        # By default we just use the bundled copy
        return False


class Distribution(Distribution):
    def has_c_libraries(self):
        return not use_system()


class build_clib(_build_clib):
    def get_source_files(self):
        files = glob.glob(here("src/libsodium/*"))
        files += glob.glob(here("src/libsodium/*/*"))
        files += glob.glob(here("src/libsodium/*/*/*"))
        files += glob.glob(here("src/libsodium/*/*/*/*"))
        files += glob.glob(here("src/libsodium/*/*/*/*/*"))
        files += glob.glob(here("src/libsodium/*/*/*/*/*/*"))

        return files

    def build_libraries(self, libraries):
        raise Exception("build_libraries")

    def check_library_list(self, libraries):
        raise Exception("check_library_list")

    def get_library_names(self):
        return ["sodium"]

    def run(self):
        if use_system():
            return

        # use Python's build environment variables
        build_env = {
            key: val
            for key, val in get_config_vars().items()
            if key in ("LDFLAGS", "CFLAGS", "CC", "CCSHARED", "LDSHARED")
            and key not in os.environ
        }
        os.environ.update(build_env)

        # Ensure our temporary build directory exists
        build_temp = os.path.abspath(self.build_temp)
        try:
            os.makedirs(build_temp)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise

        # Ensure all of our executable files have their permission set
        for filename in [
            "src/libsodium/autogen.sh",
            "src/libsodium/configure",
        ]:
            os.chmod(here(filename), 0o755)

        make_command = os.environ.get("MAKE") or "make"

        if not shutil.which(make_command):
            raise Exception(
                "ERROR: The '%s' utility is missing from PATH" % make_command
            )

        # Locate our configure script
        configure = abshere("src/libsodium/configure")

        # Run ./configure
        configure_flags = [
            "--disable-shared",
            "--enable-static",
            "--disable-debug",
            "--disable-dependency-tracking",
            "--with-pic",
        ]
        if platform.system() == "SunOS":
            # On Solaris, libssp doesn't link statically and causes linker
            # errors during import
            configure_flags.append("--disable-ssp")
        if os.environ.get("SODIUM_INSTALL_MINIMAL"):
            configure_flags.append("--enable-minimal")
        subprocess.check_call(
            [configure]
            + configure_flags
            + ["--prefix", os.path.abspath(self.build_clib)],
            cwd=build_temp,
        )

        make_args = os.environ.get("LIBSODIUM_MAKE_ARGS", "").split()
        # Build the library
        subprocess.check_call([make_command] + make_args, cwd=build_temp)

        # Check the build library
        subprocess.check_call(
            [make_command, "check"] + make_args, cwd=build_temp
        )

        # Install the built library
        subprocess.check_call(
            [make_command, "install"] + make_args, cwd=build_temp
        )


class build_ext(_build_ext):
    def run(self):
        if self.distribution.has_c_libraries():
            build_clib = self.get_finalized_command("build_clib")
            self.include_dirs.append(
                os.path.join(build_clib.build_clib, "include"),
            )
            self.library_dirs.insert(
                0,
                os.path.join(build_clib.build_clib, "lib64"),
            )
            self.library_dirs.insert(
                0,
                os.path.join(build_clib.build_clib, "lib"),
            )

        return _build_ext.run(self)


README = open("README.rst").read()
INSTALL = open("INSTALL.rst").read()
CHANGELOG = open("CHANGELOG.rst").read()


setup(
    long_description="\n".join((README, INSTALL, CHANGELOG)),
    url=nacl.__uri__,
    package_dir={"": "src"},
    packages=["nacl", "nacl.pwhash", "nacl.bindings"],
    package_data={"nacl": ["py.typed"]},
    ext_package="nacl",
    cffi_modules=["src/bindings/build.py:ffi"],
    cmdclass={"build_clib": build_clib, "build_ext": build_ext},
    distclass=Distribution,
    zip_safe=False,
)



import nox

nox.options.reuse_existing_virtualenvs = True
nox.options.default_venv_backend = "uv|virtualenv"


@nox.session
def tests(session: nox.Session) -> None:
    session.install(".[tests]")

    if session.posargs:
        tests = session.posargs
    else:
        tests = ["tests/"]

    session.run(
        "pytest",
        "-n",
        "auto",
        "--dist=worksteal",
        "--cov=nacl",
        "--cov=tests",
        "--cov-context=test",
        *tests,
    )


@nox.session
def docs(session: nox.Session) -> None:
    session.install("doc8", ".[docs]")
    tmpdir = session.create_tmp()

    session.run(
        "sphinx-build",
        "-W",
        "-b",
        "html",
        "-d",
        f"{tmpdir}/doctrees",
        "docs",
        "docs/_build/html",
    )
    session.run(
        "sphinx-build",
        "-W",
        "-b",
        "doctest",
        "-d",
        f"{tmpdir}/doctrees",
        "docs",
        "docs/_build/html",
    )
    session.run(
        "sphinx-build", "-W", "-b", "linkcheck", "docs", "docs/_build/html"
    )
    session.run("doc8", "README.rst", "docs/", "--ignore-path", "docs/_build/")


@nox.session
def meta(session: nox.Session) -> None:
    session.install("ruff", "check-manifest")
    session.run("ruff", "check", ".")
    session.run("ruff", "format", "--check", ".")
    session.run("check-manifest", ".")


@nox.session
def mypy(session: nox.Session) -> None:
    session.install(".[tests]", "mypy")

    session.run("mypy")
